@@ -25,7 +25,7 @@ __doctype__ = "restructuredtext en"
 from logilab.common.compat import chain, imap
 
 from logilab.astng._exceptions import InferenceError, NotFoundError, UnresolvableName
-from logilab.astng.nodes import Proxy_, List, Tuple, Function, If, TryExcept
+from logilab.astng._nodes import Proxy_, List, Tuple, Function, If, TryExcept
 
 
 class Proxy(Proxy_):
@@ -245,7 +245,7 @@ class Instance(Proxy):
     """a special node representing a class instance"""
     def getattr(self, name, context=None, lookupclass=True):
         try:
-            return self._proxied.instance_attr(name, context)
+            values = self._proxied.instance_attr(name, context)
         except NotFoundError:
             if name == '__class__':
                 return [self._proxied]
@@ -255,7 +255,15 @@ class Instance(Proxy):
                 if name in ('__name__', '__bases__', '__mro__'):
                     return self._proxied.local_attr(name)
                 return self._proxied.getattr(name, context)
-        raise NotFoundError(name)
+            raise NotFoundError(name)
+        # since we've no context information, return matching class members as
+        # well
+        if lookupclass:
+            try:
+                return values + self._proxied.getattr(name, context)
+            except NotFoundError:
+                pass
+        return values
 
     def igetattr(self, name, context=None):
         """infered getattr"""
@@ -279,12 +287,8 @@ class Instance(Proxy):
                 if '__builtin__.property' in attr.decoratornames():
                     for infered in attr.infer_call_result(self, context):
                         yield infered
-                elif attr.type in ('method', 'classmethod'):
-                    # XXX could get some information from the bound node:
-                    #     self (if method) or self._proxied (if class method)
-                    yield BoundMethod(attr)
                 else:
-                    yield attr
+                    yield BoundMethod(attr, self)
             else:
                 yield attr
 
@@ -316,6 +320,9 @@ class Instance(Proxy):
     def pytype(self):
         return self._proxied.qname()
 
+    def display_type(self):
+        return 'Instance of'
+
 
 class UnboundMethod(Proxy):
     """a special node representing a method not bound to an instance"""
@@ -341,9 +348,18 @@ class UnboundMethod(Proxy):
 
 class BoundMethod(UnboundMethod):
     """a special node representing a method bound to an instance"""
+    def __init__(self,  proxy, bound):
+        UnboundMethod.__init__(self, proxy)
+        self.bound = bound
+
     def is_bound(self):
         return True
 
+    def infer_call_result(self, caller, context):
+        context = context.clone()
+        context.boundnode = self.bound
+        return self._proxied.infer_call_result(caller, context)
+
 
 class Generator(Proxy):
     """a special node representing a generator"""
@@ -353,4 +369,6 @@ class Generator(Proxy):
     def pytype(self):
         return '__builtin__.generator'
 
+    def display_type(self):
+        return 'Generator'
 
