@@ -14,7 +14,7 @@
 local scope in the language definition : Module, Class, Function (and
 Lambda to some extends).
 
-Each new methods and attributes added on each class are documented
+All new methods and attributes added on each class are documented
 below.
 
 
@@ -28,6 +28,7 @@ from __future__ import generators
 
 __doctype__ = "restructuredtext en"
 
+import __builtin__
 import sys
 
 from logilab.common.compat import chain, set
@@ -35,14 +36,14 @@ from logilab.common.decorators import cached
 
 from logilab.astng import MANAGER, NotFoundError, NoDefault, \
      ASTNGBuildingException, InferenceError
-from logilab.astng.nodes import Arguments, Class, Const, Function, GenExpr, \
-     From, Lambda, Module, Name, Pass, Raise, Tuple, List, Dict, Yield, \
-     DelAttr, DelName, const_factory as cf
-from logilab.astng.utils import extend_class
-from logilab.astng.infutils import YES, InferenceContext, Instance, \
-     UnboundMethod, copy_context, unpack_infer, _infer_stmts
-from logilab.astng.nodes_as_string import as_string
+from logilab.astng._nodes import (Arguments, Class, Const, Dict, From, Function,
+     GenExpr, Lambda, List, Module, Name, Pass, Raise, Return, Tuple, Yield,
+     AssName, DelAttr, DelName, const_factory as cf, NodeNG, StmtMixIn)
 
+from logilab.astng.infutils import YES, InferenceContext, Instance, Generator, \
+     UnboundMethod, BoundMethod, copy_context, unpack_infer, _infer_stmts
+from logilab.astng.nodes_as_string import as_string
+from logilab.astng.lookup import LookupMixIn, LocalsDictMixIn
 
 def remove_nodes(func, cls):
     def wrapper(*args, **kwargs):
@@ -52,128 +53,14 @@ def remove_nodes(func, cls):
         return nodes
     return wrapper
 
-def function_to_unbound_method(func):
-    def wrapper(*args, **kwargs):
-        return [isinstance(n, Function) and UnboundMethod(n) or n
-                for n in func(*args, **kwargs)]
-    return wrapper
 
-# module class dict/iterator interface ########################################
-
-class LocalsDictMixIn(object):
-    """ this class provides locals handling common to Module, Function
-    and Class nodes, including a dict like interface for direct access
-    to locals information
-
-    /!\ this class should not be used directly /!\ it's
-    only used as a methods and attribute container, and update the
-    original class from the compiler.ast module using its dictionnary
-    (see below the class definition)
-    """
-
-    # attributes below are set by the builder module or by raw factories
-
-    # dictionary of locals with name as key and node defining the local as
-    # value
-    locals = None
-
-    def qname(self):
-        """return the 'qualified' name of the node, eg module.name,
-        module.class.name ...
-        """
-        if self.parent is None:
-            return self.name
-        return '%s.%s' % (self.parent.frame().qname(), self.name)
-
-    def frame(self):
-        """return the first parent frame node (i.e. Module, Function or Class)
-        """
-        return self
-
-    def scope(self):
-        """return the first node defining a new scope (i.e. Module,
-        Function, Class, Lambda but also GenExpr)
-        """
-        return self
-
-    def set_local(self, name, stmt):
-        """define <name> in locals (<stmt> is the node defining the name)
-        if the node is a Module node (i.e. has globals), add the name to
-        globals
-
-        if the name is already defined, ignore it
-        """
-        assert self.locals is not None, (self, id(self))
-        #assert not stmt in self.locals.get(name, ()), (self, stmt)
-        self.locals.setdefault(name, []).append(stmt)
-
-    __setitem__ = set_local
-
-    def _append_node(self, child):
-        """append a child, linking it in the tree"""
-        self.body.append(child)
-        child.parent = self
-
-    def add_local_node(self, child_node, name=None):
-        """append a child which should alter locals to the given node"""
-        if name != '__class__':
-            # add __class__ node as a child will cause infinite recursion later!
-            self._append_node(child_node)
-        self.set_local(name or child_node.name, child_node)
-
-
-    def __getitem__(self, item):
-        """method from the `dict` interface returning the first node
-        associated with the given name in the locals dictionnary
-
-        :type item: str
-        :param item: the name of the locally defined object
-        :raises KeyError: if the name is not defined
-        """
-        return self.locals[item][0]
-
-    def __iter__(self):
-        """method from the `dict` interface returning an iterator on
-        `self.keys()`
-        """
-        return iter(self.keys())
-
-    def keys(self):
-        """method from the `dict` interface returning a tuple containing
-        locally defined names
-        """
-        return self.locals.keys()
-
-    def values(self):
-        """method from the `dict` interface returning a tuple containing
-        locally defined nodes which are instance of `Function` or `Class`
-        """
-        return [self[key] for key in self.keys()]
-
-    def items(self):
-        """method from the `dict` interface returning a list of tuple
-        containing each locally defined name with its associated node,
-        which is an instance of `Function` or `Class`
-        """
-        return zip(self.keys(), self.values())
-
-    def has_key(self, name):
-        """method from the `dict` interface returning True if the given
-        name is defined in the locals dictionary
-        """
-        return self.locals.has_key(name)
-
-    __contains__ = has_key
-
-extend_class(Module, LocalsDictMixIn)
-extend_class(Class, LocalsDictMixIn)
-extend_class(Function, LocalsDictMixIn)
-extend_class(Lambda, LocalsDictMixIn)
-# GenExpr has its own locals but isn't a frame
-extend_class(GenExpr, LocalsDictMixIn)
-def frame(self):
-    return self.parent.frame()
-GenExpr.frame = frame
+def function_to_method(n, klass):
+    if isinstance(n, Function):
+        if n.type == 'classmethod':
+            return BoundMethod(n, klass)
+        if n.type != 'staticmethod':
+            return UnboundMethod(n)
+    return n
 
 def std_special_attributes(self, name, add_locals=True):
     if add_locals:
@@ -191,10 +78,10 @@ def std_special_attributes(self, name, add_locals=True):
 
 # Module  #####################################################################
 
-class ModuleNG(object):
+class ModuleNG(LookupMixIn, LocalsDictMixIn, NodeNG):
     """/!\ this class should not be used directly /!\ it's
     only used as a methods and attribute container, and update the
-    original class from the compiler.ast module using its dictionnary
+    original class from the compiler.ast module using its dictionary
     (see below the class definition)
     """
     fromlineno = 0
@@ -221,9 +108,34 @@ class ModuleNG(object):
     # names of module attributes available through the global scope
     scope_attrs = set(('__name__', '__doc__', '__file__', '__path__'))
 
+    # Module is not a Statement node but needs the replace method (see StmtMixIn)
+    def replace(self, child, newchild):
+        sequence = self.child_sequence(child)
+        newchild.parent = self
+        child.parent = None
+        sequence[sequence.index(child)] = newchild
+
+    def block_range(self, lineno):
+        """return block line numbers.
+
+        start from the "class" position whatever the given lineno
+        """
+        return self.fromlineno, self.tolineno
+
+    def scope_lookup(self, node, name, offset=0):
+        if name in self.scope_attrs and not name in self.locals:
+            try:
+                return self, self.getattr(name)
+            except NotFoundError:
+                return self, ()
+        return self._scope_lookup(node, name, offset)
+
     def pytype(self):
         return '__builtin__.module'
 
+    def display_type(self):
+        return 'Module'
+
     def getattr(self, name, context=None):
         if not name in self.special_attributes:
             try:
@@ -350,24 +262,80 @@ class ModuleNG(object):
         except AttributeError:
             return [name for name in self.keys() if not name.startswith('_')]
 
-extend_class(Module, ModuleNG)
+
+class GenExprNG(LookupMixIn, LocalsDictMixIn, NodeNG):
+    """class representing a GenExpr node"""
+
+    def frame(self):
+        return self.parent.frame()
+
 
 # Function  ###################################################################
 
-class FunctionNG(object):
+class LambdaNG(LookupMixIn, LocalsDictMixIn, NodeNG):
     """/!\ this class should not be used directly /!\ it's
     only used as a methods and attribute container, and update the
-    original class from the compiler.ast module using its dictionnary
+    original class from the compiler.ast module using its dictionary
     (see below the class definition)
     """
 
+    # function's type, 'function' | 'method' | 'staticmethod' | 'classmethod'
+    type = 'function'
+
+    def pytype(self):
+        if 'method' in self.type:
+            return '__builtin__.instancemethod'
+        return '__builtin__.function'
+
+    def display_type(self):
+        if 'method' in self.type:
+            return 'Method'
+        return 'Function'
+
+    def callable(self):
+        return True
+
+    def argnames(self):
+        """return a list of argument names"""
+        if self.args.args: # maybe None with builtin functions
+            names = _rec_get_names(self.args.args)
+        else:
+            names = []
+        if self.args.vararg:
+            names.append(self.args.vararg)
+        if self.args.kwarg:
+            names.append(self.args.kwarg)
+        return names
+
+    def infer_call_result(self, caller, context=None):
+        """infer what a function is returning when called"""
+        return self.body.infer(context)
+
+    def scope_lookup(self, node, name, offset=0):
+        if node in self.args.defaults:
+            frame = self.parent.frame()
+            # line offset to avoid that def func(f=func) resolve the default
+            # value to the defined function
+            offset = -1
+        else:
+            # check this is not used in function decorators
+            frame = self
+        return frame._scope_lookup(node, name, offset)
+
+
+class FunctionNG(LambdaNG, LookupMixIn, LocalsDictMixIn, StmtMixIn, NodeNG):
+    """/!\ this class should not be used directly /!\ it's
+    only used as a methods and attribute container, and update the
+    original class from the compiler.ast module using its dictionary
+    (see below the class definition)
+    """
+
+
     special_attributes = set(('__name__', '__doc__', '__dict__'))
     # attributes below are set by the builder module or by raw factories
 
     blockstart_tolineno = None
 
-    # function's type, 'function' | 'method' | 'staticmethod' | 'classmethod'
-    type = 'function'
 
     def set_line_info(self, lastchild):
         self.fromlineno = self.lineno
@@ -377,10 +345,12 @@ class FunctionNG(object):
         self.tolineno = lastchild.tolineno
         self.blockstart_tolineno = self.args.tolineno
 
-    def pytype(self):
-        if 'method' in self.type:
-            return '__builtin__.instancemethod'
-        return '__builtin__.function'
+    def block_range(self, lineno):
+        """return block line numbers.
+
+        start from the "class" position whatever the given lineno
+        """
+        return self.fromlineno, self.tolineno
 
     def getattr(self, name, context=None):
         """this method doesn't look in the instance_attrs dictionary since it's
@@ -409,17 +379,6 @@ class FunctionNG(object):
                 result.add(infnode.qname())
         return result
 
-    def argnames(self):
-        """return a list of argument names"""
-        if self.args.args: # maybe None with builtin functions
-            names = _rec_get_names(self.args.args)
-        else:
-            names = []
-        if self.args.vararg:
-            names.append(self.args.vararg)
-        if self.args.kwarg:
-            names.append(self.args.kwarg)
-        return names
 
     def is_bound(self):
         """return true if the function is bound to an Instance or a class"""
@@ -453,14 +412,23 @@ class FunctionNG(object):
         except StopIteration:
             return False
 
-extend_class(Function, FunctionNG)
+    def infer_call_result(self, caller, context=None):
+        """infer what a function is returning when called"""
+        if self.is_generator():
+            yield Generator(self)
+            return
+        returns = self.nodes_of_class(Return, skip_klass=Function)
+        for returnnode in returns:
+            if returnnode.value is None:
+                yield None
+            else:
+                try:
+                    for infered in returnnode.value.infer(context):
+                        yield infered
+                except InferenceError:
+                    yield YES
 
 
-# lambda nodes may also need some of the function members
-Lambda.type = 'function'
-Lambda.pytype = FunctionNG.pytype.im_func
-Lambda.argnames = FunctionNG.argnames.im_func
-
 
 def _rec_get_names(args, names=None):
     """return a list of all argument names"""
@@ -473,56 +441,6 @@ def _rec_get_names(args, names=None):
             names.append(arg.name)
     return names
 
-def format_args(self):
-    """return arguments formatted as string"""
-    result = [_format_args(self.args, self.defaults)]
-    if self.vararg:
-        result.append('*%s' % self.vararg)
-    if self.kwarg:
-        result.append('**%s' % self.kwarg)
-    return ', '.join(result)
-Arguments.format_args = format_args
-
-def default_value(self, argname):
-    """return the default value for an argument
-
-    :raise `NoDefault`: if there is no default value defined
-    """
-    i = _find_arg(argname, self.args)[0]
-    if i is not None:
-        idx = i - (len(self.args) - len(self.defaults))
-        if idx >= 0:
-            return self.defaults[idx]
-    raise NoDefault()
-Arguments.default_value = default_value
-
-def is_argument(self, name):
-    """return True if the name is defined in arguments"""
-    if name == self.vararg:
-        return True
-    if name == self.kwarg:
-        return True
-    return self.find_argname(name, True)[1] is not None
-Arguments.is_argument = is_argument
-
-def find_argname(self, argname, rec=False):
-    """return index and Name node with given name"""
-    if self.args: # self.args may be None in some cases (builtin function)
-        return _find_arg(argname, self.args, rec)
-    return None, None
-Arguments.find_argname = find_argname
-
-def _find_arg(argname, args, rec=False):
-    for i, arg in enumerate(args):
-        if isinstance(arg, Tuple):
-            if rec:
-                found = _find_arg(argname, arg.elts)
-                if found[0] is not None:
-                    return found
-        elif arg.name == argname:
-            return i, arg
-    return None, None
-
 
 def _format_args(args, defaults=None):
     values = []
@@ -539,6 +457,7 @@ def _format_args(args, defaults=None):
                 values[-1] += '=' + defaults[i-default_offset].as_string()
     return ', '.join(values)
 
+
 # Class ######################################################################
 
 def _class_type(klass):
@@ -569,10 +488,10 @@ def _iface_hdlr(iface_node):
     return True
 
 
-class ClassNG(object):
+class ClassNG(LookupMixIn, LocalsDictMixIn,  StmtMixIn, NodeNG):
     """/!\ this class should not be used directly /!\ it's
     only used as a methods and attribute container, and update the
-    original class from the compiler.ast module using its dictionnary
+    original class from the compiler.ast module using its dictionary
     (see below the class definition)
     """
     special_attributes = set(('__name__', '__doc__', '__dict__', '__module__',
@@ -610,11 +529,38 @@ class ClassNG(object):
             self.tolineno = lastchild.tolineno
         # else this is a class with only a docstring, then tolineno is (should be) already ok
 
+    def block_range(self, lineno):
+        """return block line numbers.
+
+        start from the "class" position whatever the given lineno
+        """
+        return self.fromlineno, self.tolineno
+
     def pytype(self):
         if self.newstyle:
             return '__builtin__.type'
         return '__builtin__.classobj'
 
+    def display_type(self):
+        return 'Class'
+
+    def callable(self):
+        return True
+
+    def infer_call_result(self, caller, context=None):
+        """infer what a class is returning when called"""
+        yield Instance(self)
+
+    def scope_lookup(self, node, name, offset=0):
+        if node in self.bases:
+            frame = self.parent.frame()
+            # line offset to avoid that class A(A) resolve the ancestor to
+            # the defined class
+            offset = -1
+        else:
+            frame = self
+        return frame._scope_lookup(node, name, offset)
+
     # attributes below are set by the builder module or by raw factories
 
     # a dictionary of class instances attributes
@@ -725,6 +671,8 @@ class ClassNG(object):
                 # XXX mro is read-only but that's not our job to detect that
                 return [cf(tuple(self.ancestors(recurs=True, context=context)))] + values
             return std_special_attributes(self, name)
+        # don't modify the list in self.locals!
+        values = list(values)
         for classnode in self.ancestors(recurs=False, context=context):
             try:
                 values += classnode.getattr(name, context)
@@ -733,7 +681,6 @@ class ClassNG(object):
         if not values:
             raise NotFoundError(name)
         return values
-    getattr = function_to_unbound_method(remove_nodes(getattr, DelAttr))
 
     def igetattr(self, name, context=None):
         """infered getattr, need special treatment in class to handle
@@ -755,7 +702,7 @@ class ClassNG(object):
                     else:
                         yield YES
                 else:
-                    yield infered
+                    yield function_to_method(infered, self)
         except NotFoundError:
             if not name.startswith('__') and self.has_dynamic_getattr(context):
                 # class handle some dynamic attributes, return a YES object
@@ -823,4 +770,4 @@ class ClassNG(object):
         if not found:
             raise InferenceError()
 
-extend_class(Class, ClassNG)
+
